#
# Copyright (c) 2011-2014 SUSE Linux Products GmbH
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.

package SUSE::Corral;
use strict;
use XML::Simple;
use Data::Dumper;
use Sys::Syslog qw(:standard :macros);
use Locale::gettext ();
use File::Temp;
use File::Path qw(mkpath);
use IPC::Cmd qw[run];

=head1 NAME

SUSE::Corral - compliance checking library

=head1 SYNOPSIS

  use SUSE::Corral;
  my $corral = SUSE::Corral->new();
  my $numErrors = $corral->checkRules();
  my $resultHash = $corral->getResults();

=head1 DESCRIPTION

SUSE::Corral is a compliance checking library. It compares
a set of profiles and its rules with the system. Only profiles
which matches an installed product are used for the checks.

=head1 METHODS

=over 4

=item new(options)

Initialize the corral object.

* verbose (0/1)

* debug (0/1)

* root (path to new root directory - default is "/")

* profiles (array reference with paths to profiles relative to B<root>)

* products (array reference with hashes. A hash must contain the following values of a product:B<name>,B<version>,B<release>,B<arch>,B<vendor>)

* sigkeys (array reference with paths to gpg keyfiles used to verify the profile signature relative to B<root>)

* testdata (hash reference with test hardware data - for development/tests only)

All values are optional. If I<profiles>, I<products> and I<sigkeys> are not given, the profile check uses installed products and profiles.

=cut

sub new
{
    my $pkgname = shift;
    my %opt   = @_;
    my $self  = {};
    my $profiledir = "etc/productprofiles.d";
    my $root = "/";

    $self->{gpgkeys} = [];
    $self->{gpgdir} = File::Temp::tempdir("corral-XXXXXXXX", CLEANUP => 1, TMPDIR => 1);
    chmod 0700, $self->{gpgdir};
    $self->{profilesIn} = [];
    $self->{productsIn} = [];
    $self->{profiles} = [];
    $self->{products} = [];
    $self->{result} = {};
    $self->{verbose} = 0;
    $self->{debug} = 0;
    $self->{lateVerify} = 0;
    $self->{syslogtrans} = {"critical" => LOG_CRIT,
                            "error"    => LOG_ERR,
                            "warn"     => LOG_WARNING,
                            "info"     => LOG_INFO,
                            "debug"    => LOG_DEBUG};
    openlog("SUSE_Corral", "ndelay,pid", 'user');

    # handle options
    $root = $opt{root} if ($opt{root});

    if ( exists $opt{profiles} && $opt{profiles} && ref($opt{profiles}) eq "ARRAY")
    {
        foreach my $path (@{$opt{profiles}})
        {
            push @{$self->{profilesIn}}, "$root/$path";
        }
    }
    else
    {
        # installed product profiles
        $self->{profilesIn} = [glob("$root/$profiledir/*.profile")];
    }

    if ( exists $opt{products} && $opt{products} && ref($opt{products}) eq "ARRAY")
    {
        $self->{productsIn} = $opt{products};
    }
    else
    {
        $self->{productsIn} = [glob("$root/etc/products.d/*.prod")];
    }

    if ($opt{testdata} && ref($opt{testdata}) eq "HASH")
    {
        $self->{testdata} = $opt{testdata};
    }

    # set verbose and debug option
    $self->{verbose} = 1 if ($opt{verbose});
    if( $opt{debug} )
    {
        # debug also set verbose
        $self->{verbose} = 1;
        $self->{debug} = 1;
    }

    if( $opt{sigkeys} && ref($opt{sigkeys}) eq "ARRAY")
    {
        foreach my $file (@{$opt{sigkeys}})
        {
            if (-f "$root/$file")
            {
                push @{$self->{gpgkeys}}, "$root/$file";
            }
        }
    }

    bless($self);
    return $self;
}

sub __
{
    my $self = shift;
    my $msgid = shift;
    my $domain = "subscription-tools";
    return Locale::gettext::dgettext ($domain, $msgid);
}

sub __N
{
    my $self = shift;
    my ($msgid, $msgidpl, $n) = @_;
    my $domain = "subscription-tools";
    return Locale::gettext::dngettext ($domain, $msgid, $msgidpl, $n);
}

sub __NOOP
{
    my $self = shift;
    return shift;
}

sub _printLog
{
    my $self = shift;
    my $category = shift;
    my $message  = shift;

    return if(!$self->{debug} && $category eq "debug");
    return if(!$self->{verbose} && $category eq "info");

    foreach my $line (split(/\n/, $message))
    {
        syslog($self->{syslogtrans}->{$category}, $line);
        #print STDERR "$line\n" if (exists $self->{testdata});
    }
    return;
}

sub _getProcValue()
{
    my $self = shift;
    my $file = shift;
    my $field = shift;

    if (exists $self->{testdata} && exists $self->{testdata}->{"$field"}) {
        $self->_printLog("debug", "[TEST] _getProcValue($file, $field) = ".$self->{testdata}->{"$field"});
        return $self->{testdata}->{"$field"};
    }
    open IN, "<$file" or do {
        $self->{result}->{general} = { 'type' => 'text',
            'message' => sprintf($self->__('Cannot open file: %s'), $!)};
        $self->_printLog("error", "Cannot open file: $!");
        return undef;
    };
    while (<IN>) {
        if (/$field/) {
            my @values = split ": ", $_;
            @values = split  " ", $values[1];
            $self->_printLog("debug", "_getProcValue($file, $field) = ".$values[0]);
            return $values[0];
        }
    }
    $self->_printLog("debug", "_getProcValue($file, $field) = undef");
    return undef;
}

sub _detectVirtualization()
{
    my $self = shift;

    if (exists $self->{testdata} && exists $self->{testdata}->{virtualization}) {
        $self->_printLog("debug", "[TEST] _detectVirtualization() = (".
                         $self->{testdata}->{virtualization}->[0].", ".
                         $self->{testdata}->{virtualization}->[1]);
        return @{$self->{testdata}->{virtualization}};
    }

    if(-d "/proc/xen" && ! -e "/proc/xen/xsd_port") {
        $self->_printLog("debug", "_detectVirtualization() = (true, xen)");
        return ("true", "xen");
    }
    else {
        my $out = `LANG=en_US /usr/bin/lscpu`;
        if ($? == 0 && defined $out && $out =~ /^Hypervisor vendor:\s*(.*)\s*$/m && defined $1)
        {
            $self->_printLog("debug", "_detectVirtualization() = (true, lc($1))");
            return ("true", lc($1));
        }

        # detect VMWare
        $out = `LANG=en_US /usr/sbin/hwinfo --gfxcard`;
        if ($? == 0 && defined $out && $out =~ /Vendor:.+VMWare/ig)
        {
            $self->_printLog("debug", "_detectVirtualization() = (true, vmware)");
            return ("true", "vmware");
        }

        # detect VirtualBox
        $out = `LANG=en_US /usr/sbin/hwinfo --bios`;
        if ($? == 0 && defined $out && $out =~ /Product:.+VirtualBox/ig)
        {
            $self->_printLog("debug", "_detectVirtualization() = (true, virtualbox)");
            return ("true", "virtualbox");
        }

        # detect PowerVM
        $out = `LANG=en_US /usr/sbin/hwinfo --cpu`;
        if ($? == 0 && defined $out && $out =~ /Vendor:.+PowerVM\sLx86/ig)
        {
            $self->_printLog("debug", "_detectVirtualization() = (true, powervm_lx86)");
            return ("true", "powervm_lx86");
        }
    }
    $self->_printLog("debug", "_detectVirtualization() = (false, '')");
    return ("false", "");
}

sub _getCPUValues
{
    my $self = shift;
    my $sockets = 0;
    my $coresps = 0;
    my $threadspc = 0;

    if (exists $self->{testdata} && exists $self->{testdata}->{CPU})
    {
        $self->_printLog("debug", "[TEST] _getCPUValues() = ".
                          $self->{testdata}->{CPU}->[0].",".
                          $self->{testdata}->{CPU}->[1].",".
                          $self->{testdata}->{CPU}->[2].",");
        return @{$self->{testdata}->{CPU}};
    }

    my $out = `LANG=en_US /usr/bin/lscpu`;
    if ($? == 0 && $out)
    {
        foreach my $line (split(/\n/, $out))
        {
            if($line =~ /^Thread.+per\score:\s*(\d+)\s*$/)
            {
                $threadspc = int($1);
            }
            if($line =~ /^Core.+per\ssocket:\s*(\d+)\s*$/)
            {
                $coresps = int($1);
            }
            if($line =~ /^CPU\ssocket.+:\s*(\d+)\s*$/)
            {
                $sockets = int($1);
            }
        }
    }
    $self->_printLog("debug", "_getCPUValues() = ".$sockets.",".($sockets * $coresps).",".($sockets * $coresps * $threadspc));
    return ($sockets, ($sockets * $coresps), ($sockets * $coresps * $threadspc));
}

sub _getValueDescription
{
    my $self = shift;
    my $id = shift;

    if ($id eq "cpu-cores")
    {
        return $self->__NOOP("Number of CPU cores");
    }
    elsif ($id eq "cpu-sockets")
    {
        return $self->__NOOP("Number of CPU sockets");
    }
    elsif ($id eq "cpu-threads")
    {
        return $self->__NOOP("Number of CPU threads");
    }
    elsif ($id eq "arch")
    {
        return $self->__NOOP("Architecture");
    }
    elsif ($id eq "ram")
    {
        return $self->__NOOP("Amount of RAM in kb");
    }
    elsif ($id eq "virtual")
    {
        return $self->__NOOP("Is virtualized");
    }
    elsif ($id eq "hypervisor")
    {
        return $self->__NOOP("Hypervisor used");
    }
    return $id;
}

sub _getValue
{
    my $self = shift;
    my $id = shift;

    if ($id eq "cpu-cores")
    {
        my ($sockets, $cores, $threads) = $self->_getCPUValues();
        return $cores;
    }
    elsif ($id eq "cpu-sockets")
    {
        my ($sockets, $cores, $threads) = $self->_getCPUValues();
        return $sockets;
    }
    elsif ($id eq "cpu-threads")
    {
        my ($sockets, $cores, $threads) = $self->_getCPUValues();
        return $threads;
    }
    elsif ($id eq "arch")
    {
        if (exists $self->{testdata} && exists $self->{testdata}->{arch}) {
            $self->_printLog("debug", "[TEST] arch = ".$self->{testdata}->{arch});
            return $self->{testdata}->{arch};
        }
        my $output = qx(rpm --eval="%{_arch}");
        chomp($output);
        $self->_printLog("debug", "arch = $output");
        return $output;
    }
    elsif ($id eq "ram")
    {
        return $self->_getProcValue("/proc/meminfo", "MemTotal");
    }
    elsif ($id eq "virtual")
    {
        my($isVirtual, $hypervisor) = $self->_detectVirtualization();
        return $isVirtual;
    }
    elsif ($id eq "hypervisor")
    {
        my($isVirtual, $hypervisor) = $self->_detectVirtualization();
        return $hypervisor;
    }

    return undef;
}

sub _checkRangeRule
{
    my $self = shift;
    my $rule = shift;

    my $val = $self->_getValue($rule->{id});
    my $err = 0;

    if (defined $rule->{max})
    {
        $self->_printLog("debug", "Checking: ".$self->_getValueDescription($rule->{id})." Maximum: ".$rule->{max});
        if ($val > $rule->{max}) {
            $self->_printLog("info",
                             "  * ".$self->_getValueDescription($rule->{id})." (current value: '$val'); must not exceed '".$rule->{max}."'");
            $err = 1;
            # Translator: get a tagged value
            my $message = $self->__($self->_getValueDescription($rule->{id}));
            # Translator: (<is value>) exceeds <should value>
            $message   .= sprintf($self->__(" (current value: '%d'); must not exceed '%d'"), $val, $rule->{max});
            $self->{result}->{$rule->{id}} = { 'type' => 'range', max => $rule->{max}, is => $val,
                                               'message' => $message };
        }
    }
    return $err;
}

sub _checkBooleanRule
{
    my $self = shift;
    my $rule = shift;

    my $val = $self->_getValue($rule->{id});
    my $err = 0;

    if (defined $rule->{value})
    {
        $self->_printLog("debug", "Checking: ".$self->_getValueDescription($rule->{id})." is ".$rule->{value});
        if (lc($val) ne lc($rule->{value}))
        {
            $self->_printLog("info",
                             "  * ".$self->_getValueDescription($rule->{id})." (current value: '$val'); must be: '".$rule->{value}."'");
            $err = 1;
            # Translator: get a tagged value
            my $message = $self->__($self->_getValueDescription($rule->{id}));
            # Translator: (<is value>) is not <should value>
            $message   .= sprintf($self->__(" (current value: '%s'); must be: '%s'"), $val, $rule->{value});

            $self->{result}->{$rule->{id}} = { 'type' => 'boolean', should => $rule->{value}, is => $val,
                                               'message' => $message };
        }
    }
    return $err;
}

sub _checkEnumRule
{
    my $self = shift;
    my $rule = shift;
    my $val = $self->_getValue($rule->{id});

    $self->_printLog("debug", "Checking: ".$self->_getValueDescription($rule->{id})." is one of: ".join(", ", @{$rule->{v}}));

    my $err = 0;
    my $valid = 0;
    foreach my $allowed (@{$rule->{v}}) {
        $valid = 1 if lc($allowed) eq lc($val);
    }
    if ( !$valid ) {
        $self->_printLog("info",
                         "  *  ".$self->_getValueDescription($rule->{id})." (current value: '$val'); must be one of: '".join(", ", @{$rule->{v}})."'");
        $err = 1;
        # Translator: get a tagged value
        my $message = $self->__($self->_getValueDescription($rule->{id}));
        # Translator: (<is value>) is not one of <allowed values>
        $message   .= sprintf($self->__(" (current value: '%s'); must be one of: '%s'"), $val, join(", ", @{$rule->{v}}));

        $self->{result}->{$rule->{id}} = { 'type' => 'enum', oneof => $rule->{v}, is => $val,
                                           'message' => $message };
    }
    return $err;
}

sub _checkRule
{
    my $self = shift;
    my $rule = shift;
    my $err = 0;
    if ($rule->{type} eq "range") {
        $err = $self->_checkRangeRule($rule);
    }
    elsif ($rule->{type} eq "enum") {
        $err = $self->_checkEnumRule($rule);
    }
    elsif ($rule->{type} eq "boolean") {
        $err = $self->_checkBooleanRule($rule);
    }
    return $err;
}

sub _lateVerifyProfile
{
    my $self = shift;
    my $profilefile = shift;
    my $productfiles = shift;
    my @pr = ();
    my $origLang = $ENV{'LANG'};

    return 0 if(! $self->{lateVerify} );

    if( $productfiles eq "all" )
    {
        push @pr, @{$self->{products}};
    }
    elsif( -f $productfiles )
    {
        push @pr, $productfiles;
    }
    else
    {
        $self->_printLog("error", "$productfiles: is not a file");
        return 1;
    }

    # lets create a new empty tmpdir
    # we want to use a keyring containing only the keys of the product rpms
    $self->{gpgdir} = File::Temp::tempdir("corral-XXXXXXXX", CLEANUP => 1, TMPDIR => 1);
    chmod 0700, $self->{gpgdir};
    mkpath( $self->{gpgdir}."/tmp" );

    $ENV{'LANG'} = "en_US.UTF-8";
    foreach my $productfilepath (@pr)
    {
        $self->_printLog("debug", "Search signature key of ".$productfilepath);
        if ( -f $productfilepath )
        {
            my $cmd = ["/bin/rpm", "-qfi", $productfilepath];
            my( $success, $error_code, $full_buf, $stdout_buf, $stderr_buf ) =
                run( command => $cmd,
                     verbose => 0 );
            foreach my $line (@{$stdout_buf})
            {
                if ($line =~ /^Signature.+Key\sID\s[0-9a-f]{8}([0-9a-f]{8})\s*$/m && $1)
                {
                    $self->_printLog("debug", "Key ID: $1");
                    $cmd = ["/usr/bin/gpg2",
                            '--no-default-keyring',
                            '--quiet',
                            '--no-tty',
                            '--batch',
                            '--no-greeting',
                            '--no-permission-warning',
                            '--status-fd',
                            '2',
                            '--homedir', '/usr/lib/rpm/gnupg/',
                            '--export', '--armor',
                            '--output', "$self->{gpgdir}/tmp/gpg-pubkey-$1.key",
                            "$1"];
                    my( $success, $error_code, $full_buf, $stdout_buf, $stderr_buf ) =
                    run( command => $cmd,
                         verbose => 0 );
                    $self->_printLog("debug", "GPG Key: ".join('', @{$stderr_buf}));
                }
            }
        }
    }
    $ENV{'LANG'} = $origLang;
    $self->{gpgkeys} = [glob("$self->{gpgdir}/tmp/*.key")];
    $self->_printLog("debug", "Test against the following GPG Keys:".Data::Dumper->Dump([$self->{gpgkeys}]));
    # we ignore import errors here
    # if the correct key was not imported, verify will fail.
    $self->_importGPGKey();
    if( ! defined $self->_verifyFile($profilefile) )
    {
        return 1;
    }
    return 0;
}

sub _matchProduct
{
    my $self = shift;
    my $product = shift;
    my $missmatches = 0;

    #$self->_printLog("debug", "PRODUCT: ".Data::Dumper->Dump([$product]));

    if( !defined $product || (ref($product) eq "HASH" && !(scalar %{$product})))
    {
        # no product defined;
        # this means this profile applies to all products
        $self->_printLog("debug", "Profile applies to all products");
        return "all";
    }
    if (!exists $product->{name} || !$product->{name}) {
        $self->_printLog("error", "Invalid Profile: no product name  specified.");
        $self->{result}->{general} = { 'type' => 'text',
            'message' => $self->__('Invalid Profile: no product name specified.')};
        return undef;
    }
    $self->_printLog("debug", "Checking product ".$product->{name});
    foreach my $instProd (@{$self->{products}}) {
        if ($product->{name} eq $instProd->{name}) {
            foreach my $key (keys %$product)
            {
                next if ($key eq "name");
                if (! exists $instProd->{$key}  ||
                    ! defined $instProd->{$key} ||
                    $instProd->{$key} ne $product->{$key})
                {
                    $self->_printLog("debug", "  * Product missmatch: $key ($product->{$key}) does not match the installed product");
                    $missmatches++;
                }
            }
            if ($missmatches > 0)
            {
                # this product does not match, reset and continue search
                $missmatches = 0;
            } else {
                # match found; abort here
                $self->_printLog("debug", "Matching product found");
                return $instProd->{productfilepath};
            }
        }
    }
    # no match found
    $self->_printLog("debug", "No matching product found");
    return undef;
}

sub _testSystemTools
{
    my $self = shift;
    my $err = 0;

    if (exists $self->{testdata} && $self->{testdata})
    {
        # test systems may not have sys mounted and than some tools
        # do not work. If we work with testdata simply skip this test
        return 0;
    }

    my $out = `LANG=en_US /usr/bin/lscpu`;
    if ($? != 0 || !$out)
    {
        $self->_printLog("error", "lscpu command failed!");
        $err++;
    }

    $out = `LANG=en_US /usr/sbin/hwinfo --cpu`;
    if ($? != 0 || !$out)
    {
        $self->_printLog("error", "hwinfo command failed!");
        $err++;
    }

    if (!-r "/proc/meminfo")
    {
        $self->_printLog("error", "Unable to read meminfo");
        $err++;
    }
    if($err)
    {
        $self->{result}->{general} = { 'type' => 'text',
            'message' => $self->__('Required commands to check the compliance are not available or do not function.')};
    }
    return $err;
}

sub _importGPGKey
{
    my $self = shift;
    my $origLang = $ENV{'LANG'};

    if( @{$self->{gpgkeys}} == 0 )
    {
        # FIXME: find the key on the system
        $self->_printLog("debug", "no key specified. Set lateVerify.");
        $self->{lateVerify} = 1;
        return 0;
    }

    $ENV{'LANG'} = "en_US.UTF-8";
    foreach my $key (@{$self->{gpgkeys}})
    {
        my $buffer;
        my $cmd = ["/usr/bin/gpg2",
                   '--no-default-keyring',
                   '--quiet',
                   '--no-tty',
                   '--batch',
                   '--no-greeting',
                   '--no-permission-warning',
                   '--status-fd',
                   '1',
                   '--homedir',
                   $self->{gpgdir},
                   '--import',
                   $key];
        if(! scalar run( command => $cmd,
                         verbose => 0,
                         buffer  => \$buffer ) )
        {
            $ENV{'LANG'} = $origLang;
            $self->{result}->{general} = { 'type' => 'text',
            'message' => $self->__('Import of at least one GPG key failed.')};
            $self->_printLog("error", "Import GPG key failed: $buffer");
            return 1;
        }
        $self->_printLog("debug", "Key import successfull: $buffer");
    }
    $ENV{'LANG'} = $origLang;
    return 0;
}

sub _decryptFile
{
    my $self = shift;
    my $file = shift;
    my $origLang = $ENV{'LANG'};

    if(! -f $file)
    {
        $self->_printLog("debug", "$file: is not a file");
        return undef;
    }
    $self->_printLog("debug", "Decrypt: $file");

    $ENV{'LANG'} = "en_US.UTF-8";
    my $cmd = ["/usr/bin/gpg2",
               '--no-default-keyring',
               '--quiet',
               '--no-tty',
               '--batch',
               '--no-greeting',
               '--status-fd',
               '2',
               '--homedir',
               $self->{gpgdir},
               '--decrypt',
               '--skip-verify',
               $file];
    my( $success, $error_code, $full_buf, $stdout_buf, $stderr_buf ) =
                run( command => $cmd,
                     verbose => 0 );

    $ENV{'LANG'} = $origLang;
    if(!$success)
    {
        $self->{result}->{general} = { 'type' => 'text',
                'message' => $self->__('The profile cannot be processed.')};
        $self->_printLog("error", "Decrypt GPG profile failed: $error_code $stderr_buf");
        return undef;
    }
    $self->_printLog("debug", "Decrypt successfull: ".join("", @{$full_buf}));
    return join("", @{$stdout_buf});
}

sub _verifyFile
{
    my $self = shift;
    my $file = shift;
    my $origLang = $ENV{'LANG'};

    if(! -f $file)
    {
        $self->_printLog("debug", "$file: is not a file");
        return undef;
    }
    $self->_printLog("debug", "Verify: $file");

    my $buffer;
    my $cmd = ["/usr/bin/gpg2",
               '--no-default-keyring',
               '--quiet',
               '--no-tty',
               '--batch',
               '--no-greeting',
               '--status-fd',
               '1',
               '--homedir',
               $self->{gpgdir},
               '--verify',
               $file];
    $ENV{'LANG'} = "en_US.UTF-8";
    if(! scalar run( command => $cmd,
                     verbose => 0,
                     buffer  => \$buffer ) )
    {
        $ENV{'LANG'} = $origLang;
        $self->{result}->{general} = { 'type' => 'text',
                'message' => $self->__('Verifying the signature of the profile failed.')};
        $self->_printLog("error", "Verifying signature of the profile failed: $buffer");
        return undef;
    }
    $ENV{'LANG'} = $origLang;
    $self->_printLog("debug", "verify successfull: $buffer");

    return $self->_decryptFile($file);
}

sub _parseProductAndProfiles
{
    my $self = shift;
    my $xml = new XML::Simple;

    # reading profiles and products
    foreach my $profile (@{$self->{profilesIn}})
    {
        if(! -f $profile)
        {
            $self->{result}->{general} = { 'type' => 'text',
                'message' => sprintf($self->__("Profile not found: %s"), $profile)};
            $self->_printLog("error", "Profile not found: $profile");
            return 1;
        }

        my $p = undef;
        if(! $self->{lateVerify} )
        {
            $p = $self->_verifyFile($profile);
            return 1 if(!$p);
        }
        else
        {
            $p = $self->_decryptFile($profile);
        }
        my $parsed = $xml->XMLin($p, ForceArray => ["v", "rule"], KeyAttr => [], GroupTags => { rules => 'rule' });
        $parsed->{profilefilepath} = $profile;
        # workaround empty rules
        $parsed->{check}->{rules} = [] if(ref($parsed->{check}->{rules}) ne "ARRAY");
        push(@{$self->{profiles}}, $parsed);
    }
    foreach my $product (@{$self->{productsIn}})
    {
        if(ref($product) eq "HASH")
        {
            if ($product->{name} &&
                exists $product->{version} &&
                defined $product->{version} &&
                $product->{version} ne "" &&
                exists $product->{release} &&
                defined $product->{release} &&
                $product->{arch} &&
                $product->{vendor})
            {
                if(! $product->{baseversion} )
                {
                    my ($bv, $drop) = split(/\./, $product->{version}, 2);
                    $product->{baseversion} = $bv;
                }
                # This is a fake path to indicate that we are called from inst-sys
                $product->{productfilepath} = "/content";
                push @{$self->{products}}, $product;
            }
            else
            {
                $self->{result}->{general} = { 'type' => 'text',
                    'message' => $self->__("Invalid Product definition.")};
                $self->_printLog("error", "Invalid Product definition: ".Data::Dumper->Dump([$product]));
                return 1;
            }
        }
        elsif(-f $product)
        {
            my $parsed = $xml->XMLin($product, KeyAttr => []);

            $parsed->{productfilepath} = $product;
            if(exists $parsed->{version} && ref($parsed->{version}) eq "HASH")
            {
                # convert <version ver="11.1" rel="1.2.0"/> into
                # 'version' => 11.1 and 'release' => 1.2.0
                $parsed->{release} = $parsed->{version}->{rel};
                $parsed->{version} = $parsed->{version}->{ver};
            }

            if ((! exists $parsed->{baseversion}  ||
                ! defined $parsed->{baseversion} ) &&
                exists $parsed->{version} &&
                $parsed->{version})
            {
                # if baseversion is not a key calculate it from the version
                my ($bv, $drop) = split(/\./, $parsed->{version}, 2);
                $parsed->{baseversion} = $bv;
            }
            push(@{$self->{products}}, $parsed);
        }
        else
        {
            $self->{result}->{general} = { 'type' => 'text',
                'message' => $self->__("No Products found")};
            $self->_printLog("error", "No Products found.");
            return 1;
        }
    }
    return 0;
}

=item checkRules()

Run the check. Returns the number of errors found.

=cut

sub checkRules
{
    my $self = shift;
    my $err = 0;

    $err = $self->_testSystemTools();
    if($err)
    {
        $self->_printLog("critical",
                         "Compliance check failed. The product profile does not match the system.");
        return $err;
    }
    $err = $self->_importGPGKey();
    if($err)
    {
        $self->_printLog("critical",
                         "Compliance check failed. The product profile does not match the system.");
        return $err;
    }
    $err = $self->_parseProductAndProfiles();
    if($err)
    {
        $self->_printLog("critical",
                         "Compliance check failed. The product profile does not match the system.");
        return $err;
    }
    foreach my $profile (@{$self->{profiles}})
    {
        my $e = 0;
        # test only profiles which match installed products
        my $match = $self->_matchProduct($profile->{match});
        next if( ! defined $match );

        $e += $self->_lateVerifyProfile($profile->{profilefilepath}, $match);
        if(! $e)
        {
            foreach my $rule (@{$profile->{check}->{rules}})
            {
                $e += $self->_checkRule($rule);
            }
        }
        if($e > 0)
        {
            # on error set the action
            # FIXME: multiple different actions overwrite others
            $self->{result}->{action} = $profile->{check}->{action};
        }
        $err += $e;
    }
    if($err)
    {
        $self->_printLog("critical",
                         "Compliance check failed. The product profile does not match the system.");
    }
    return $err
}

=item getResults()

Returns the result hash with informations about the errors.

Example:

  $VAR1 = {
            'action' => 'warn',
            'arch' => {
                      'oneof' => [
                                   's390x',
                                   'i586'
                                 ],
                      'type' => 'enum',
                      'is' => 'x86_64',
                      'message' => 'Architecture(x86_64) is not one of: s390x, i586'
                    },
            'cpu-cores' => {
                           'max' => 1,
                           'type' => 'range',
                           'is' => 2,
                           'message' => 'Number of CPU cores(2) exceeds 1'
                         },
            'ram' => {
                     'max' => 3597436,
                     'type' => 'range',
                     'is' => 5992720,
                     'message' => 'Amount of RAM(5992720) exceeds 3597436'
                   }
          };

=cut

sub getResults
{
    my $self = shift;
    return $self->{result};
}

=back

=head1 PRODUCT PROFILE

A product profile has two main sections. The B<match> section defines a product
for which this profile applies to. If no installed product matches,
the checks are skipped.

For installed products it compares the elements in B<match> with the corresponding
elements of the installed products (/etc/products.d/*.prod).

The second main section is B<check>. The B<check> section has one set of rules
which must apply to the system where this product gets installed.
Supported are checks for I<cpu-sockets>, I<cpu-cores>, I<cpu-threads>,
I<ram>, I<architecture>, I<physical/virtual machine> and I<hypervisor>.
The boolean and enum rules tests are case insensitive.

The profile has three types of checks:

* range: check for a maximal number. Used for I<cpu-sockets>, I<cpu-cores>, I<cpu-threads> and I<ram>.
The value for I<ram> is given in kiB.

* enum: check for one of the given list. Used for I<architecture> and I<hypervisor>

* boolean: true or false. Used for I<physical/virtual machine>

I<action> defines what should happen if one or more rules do not apply
to the system. Currently the only allowed value for I<action> is B<warn>.

In the installed system profiles are read from /etc/productprofiles.d/*.profile

Example:

  <product-profile>
    <match>
      <name>SUSE_SLES</name>
      <vendor>SUSE LINUX Products GmbH, Nuernberg, Germany</vendor>
      <baseversion>11</baseversion>
    </match>
    <check>
      <rules>
        <rule type="range" id="cpu-sockets" max="1"/>
        <rule type="range" id="cpu-cores" max="1"/>
        <rule type="range" id="cpu-threads" max="1"/>
        <rule type="range" id="ram" max="3597436"/>
        <rule type="enum" id="arch">
           <v>x86_64</v>
           <v>i586</v>
        </rule>
        <rule type="boolean" id="virtual" value="true"/>
        <rule type="enum" id="hypervisor">
            <v>vmware</v>
        </rule>
      </rules>
      <action>warn</action>
    </check>
  </product-profile>

The product profile has to be signed. It has to be signed with the same GPG key
as the package which provides the product file which the profile matches.
The signature and the profile has to be in the same file (not detached)
(gpg -s --clearsign profile). The filename suffix must be .profile .


=head1 AUTHORS

Duncan Mac-Vicar Prett, Michael Calmer

=head1 LICENSE

Copyright 2011-2014 SUSE LINUX Products GmbH, Nuernberg, Germany.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=cut

1;

