#
# Copyright (c) 2011-2014 SUSE Linux Products GmbH
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.

package SUSE::RegistrationStatus;
use strict;
use File::Basename;
use XML::Simple;
use Data::Dumper;
use URI;
use Config::IniFiles;
use JSON;
use Date::Parse;

=head1 NAME

SUSE::RegistrationStatus - checking if a system is registered

=head1 SYNOPSIS

  use SUSE::RegistrationStatus;
  my $reg = SUSE::RegistrationStatus->new();
  my $error = $reg->isRegistered();
  my $details = $reg->details();

=head1 DESCRIPTION

SUSE::RegistrationStatus is a registration checking library. It
find out, if a system is registered against some registration service
and display the information about the subscription status if available.

=head1 METHODS

=over 4

=item new()

Create a new RegistrationStatus object.

=cut

# constructor
sub new
{
    my $pkgname = shift;
    my %opt   = @_;
    my $self  = {};
    $self->{registration_status} = undef;

    # possible: NCC, SCC, SMT, SUMA
    $self->{registration_server_type} = undef;
    bless($self);

    my $xml = new XML::Simple;
    my @result;
    my $statusfile = "/var/lib/suseRegister/registration-status.xml";

    # installed products
    my @prod_files = glob("/etc/products.d/*.prod");
    foreach my $prod_file (@prod_files)
    {
        my $prod = $xml->XMLin($prod_file);
        my $prod_name = $prod->{name};
        $prod_name = $prod_name . "-" . $prod->{version};
        # $prod_name = $prod_name . "-" . $prod->{release};
        $prod_name = $prod_name . "-" . $prod->{arch};
        push(@{$self->{installed_products}}, $prod_name);
    }

    if(-x "/usr/bin/suse_register")
    {
        # registration status
        # The status file may exist but is empty. Try only to parse it
        # if it is not empty.
        if( -s $statusfile )
        {
            $self->__parseStatus($statusfile);

            # if the status file is older than 24 hours, try to get a new one.
            if((int($self->{registration_status}->{generated}) + 86400) < time)
            {
                `/usr/bin/suse_register --batch >/dev/null 2>&1`;
                $self->{registration_server_type} = 'NCC';
            }
            $self->__parseStatus($statusfile);
        }
    }
    elsif(-x "/usr/sbin/SUSEConnect")
    {
        my $status = `/usr/sbin/SUSEConnect --status`;
        $self->{registration_server_type} = 'SCC';
        $self->__parseSCCStatus($status);
    }

    # find out the hostname of the update server
    $self->{srUpdateServerHostname} = undef;
    if (-e "/etc/suseRegister.conf" && $self->{registration_server_type} == 'NCC')
    {
        open(CNF, "< /etc/suseRegister.conf") and do
        {
            while(<CNF>)
            {
                if($_ =~ /^\s*#/)
                {
                next;
                }
                elsif($_ =~ /^url\s*=\s*(\S*)\s*/ && defined $1 && $1 ne "")
                {
                    my $uri = URI->new($1);
                    if($uri->host eq "secure-www.novell.com")
                    {
                        $self->{srUpdateServerHostname} = "nu.novell.com";
                    }
                    else
                    {
                        $self->{srUpdateServerHostname} = $uri->host;
                        $self->{registration_server_type} = 'SMT';
                    }
                    last;
                }
            }
        };
    }
    if (-e "/etc/SUSEConnect" && $self->{registration_server_type} == 'SCC')
    {
        # default to scc case
        $self->{srUpdateServerHostname} = "updates.suse.com";
        open(CNF, "< /etc/SUSEConnect") and do
        {
            while(<CNF>)
            {
                if($_ =~ /^\s*#/)
                {
                    next;
                }
                elsif($_ =~ /^\s*url\s*:\s*(\S*)\s*/ && defined $1 && $1 ne "")
                {
                    my $uri = URI->new($1);
                    if($uri->host eq "scc.suse.com")
                    {
                        $self->{srUpdateServerHostname} = "updates.suse.com";
                    }
                    else
                    {
                        $self->{srUpdateServerHostname} = $uri->host;
                        $self->{registration_server_type} = 'SMT';
                    }
                    last;
                }
            }
        };
    }

    # find out SUSE Manager update server
    $self->{smUpdateServerHostname} = undef;
    if (-e "/etc/sysconfig/rhn/systemid" && -e "/etc/sysconfig/rhn/up2date" )
    {
        open(CNF, "< /etc/sysconfig/rhn/up2date") and do
        {
            while(<CNF>)
            {
                if($_ =~ /^\s*#/)
                {
                next;
                }
                elsif($_ =~ /^serverURL\s*=\s*(\S*)\s*/ && defined $1 && $1 ne "")
                {
                    my $uri = URI->new($1);
                    $self->{smUpdateServerHostname} = $uri->host;
                    $self->{registration_server_type} = 'SUMA';
                    last;
                }
            }
        };
    }

    # read repos
    $self->{repoHostNames} = [];
    $self->{hasSMRepos} = 0;
    my @repos = glob("/etc/zypp/repos.d/*.repo");
    foreach my $repo (@repos)
    {
        my $cfg = Config::IniFiles->new( -file => $repo);
        foreach my $section ($cfg->Sections())
        {
            if($cfg->val($section, "baseurl"))
            {
                if($cfg->val($section, "baseurl") =~ /^plugin:spacewalk\?/)
                {
                    $self->{hasSMRepos} = 1;
                }
                else
                {
                    my $uri = URI->new($cfg->val($section, "baseurl"));
                    if(grep {$_ eq $uri->scheme} ("http", "https", "ftp"))
                    {
                        push @{$self->{repoHostNames}}, $uri->host;
                    }
                }
            }
        }
    }
    $self->{repoHostNames} = [keys %{{ map { $_ => 1 } @{$self->{repoHostNames}} }}];

    #print "SELF: ".Data::Dumper->Dump([$self])."\n";
    return $self;
}

=item isRegistered()

Return 1 if the sytem is registered against a registration server and
has repositories from this server. Otherwise it returns 0.

=cut

sub isRegistered
{
    my $self = shift;
    my $isRegistered = 0;

    # test for suseRegister
    if($self->{srUpdateServerHostname} &&
       grep {$_ eq $self->{srUpdateServerHostname}} @{$self->{repoHostNames}})
    {
        $isRegistered = 1;
    }

    # test for SUSE Manager
    if(!$isRegistered && $self->{smUpdateServerHostname} && $self->{hasSMRepos})
    {
        $isRegistered = 1;
    }
    return $isRegistered;
}

=item details()

Returns a text with details about the registration status.

=cut

sub details
{
    my $self = shift;
    my @result = ();

    if(!$self->isRegistered())
    {
        push @result, $self->__("The system is not registered.\n");
    }
    else
    {
        if($self->{smUpdateServerHostname} && $self->{hasSMRepos})
        {
            push @result, sprintf($self->__("The system is registered against %s.\n"), $self->{smUpdateServerHostname});
            push @result, $self->__("Please check, if the system has access to all required update repositories.\n");
            push @result, $self->__("Installed Products:\n");
            foreach my $p ($self->installedProducts())
            {
                push @result, "* $p\n";
            }
        }

        if($self->{srUpdateServerHostname} &&
            grep {$_ eq $self->{srUpdateServerHostname}} @{$self->{repoHostNames}})
        {
            push @result, sprintf($self->__("The system is registered against %s\n"), $self->{srUpdateServerHostname});
            if($self->{registration_server_type} eq "SMT")
            {
                push @result, $self->__("Please check, if the system has access to all required update repositories.\n");
            }
            push @result, $self->__("Product status:\n");
            # TRANSLATORS: the product is "not registered"
            my $status_message = $self->__("not registered");

            if(!$self->registrationStatus())
            {
                # TRANSLATORS: the registration status of the checked product is "unknown"
                $status_message = $self->__("unknown");
            }

            foreach my $p ($self->installedProducts())
            {
                my $status = $self->registrationStatusFor($p);
                if($status && exists $status->{subscription})
                {
                    if(lc($status->{subscription}->{status}) eq "active" &&
                        int($status->{subscription}->{expiration}) < time)
                    {
                        # the status file could be old
                        # let's check the expiration time
                        $status->{subscription}->{status} = "EXPIRED";
                    }
                    # TRANSLATORS: The product is registered
                    $status_message = $self->__("registered")."; ".$self->_getSubString($status->{subscription}->{status},
                                                                                        $status->{subscription}->{type});
                }
                elsif($status)
                {
                    if(uc($status->{errorcode}) eq "OK")
                    {
                        # TRANSLATOR: The product is registered
                        $status_message = $self->__("registered");
                    }
                }
                push @result, "* $p : $status_message\n";
            }
        }
    }
    return join("", @result);
}

=item registrationStatus()

Returns the registration status structure
 {
   'generated' => '1303563395',
   'productstatus' => [
      {
        'errorcode' => 'OK',
        'version' => '1.2',
        'arch' => 'x86_64',
        'release' => 'cd',
        'message' => 'No errors.',
        'product' => 'SUSE-Manager-Server',
        'subscription' => {
          'status' => 'ACTIVE',
          'expiration' => '1555428677',
          'type' => 'FULL'
         },
       'result' => 'success'
       }
    ]
 };

=cut

sub registrationStatus
{
    my $self = shift;
    #print "STATUS:".Data::Dumper->Dump([$self->{registration_status}])."\n";
    return $self->{registration_status};
}

=item I<registrationStausFor(product)>

Returns the registration status for an specific
product. The product is identified by <name>-<version>-<arch>.
This method return undef in case the product was not found.

 {
   'errorcode' => 'OK',
   'version' => '1.2',
   'arch' => 'x86_64',
   'release' => 'cd',
   'message' => 'No errors.',
   'product' => 'SUSE-Manager-Server',
   'subscription' => {
     'status' => 'ACTIVE',
     'expiration' => '1555428677',
     'type' => 'FULL'
   },
   'result' => 'success'
 };

 Usage:
 $status->registrationStatusFor("productname-1.0-x86_64")

=cut

sub registrationStatusFor
{
    my $self = shift;
    my $prod_name = shift;
    my $full_status = $self->registrationStatus();

    if (exists $full_status->{productstatus} && defined $full_status->{productstatus}) {
        my $status = $full_status->{productstatus};
        foreach my $prod (@$status) {
            my $full_name = $prod->{product} . "-" . $prod->{version} . "-" . $prod->{arch};
            if ($prod_name eq $full_name)
            {
                return $prod;
            }
        }
    }
    return undef;
}

=item installedProduct()

Return an array of the installed products in the form
<name>-<version>-<arch>

=cut

sub installedProducts
{
    my $self = shift;
    return @{$self->{installed_products}};
}

sub _getSubString
{
    my $self   = shift;
    my $status = shift || ""; # active, expired
    my $type   = shift || ""; # full, eval, prov

    $status = lc($status);
    $type   = lc($type);

    if($status eq "active" && $type eq "full")
    {
        return $self->__("Subscription is active");
    }
    elsif($status eq "active" && $type eq "eval")
    {
        return $self->__("Evaluation subscription is active");
    }
    elsif($status eq "active" && $type eq "prov")
    {
        return $self->__("Provisional subscription is active");
    }
    elsif($status eq "expired" && $type eq "full")
    {
        return $self->__("Subscription is expired");
    }
    elsif($status eq "expired" && $type eq "eval")
    {
        return $self->__("Evaluation subscription is expired");
    }
    elsif($status eq "expired" && $type eq "prov")
    {
        return $self->__("Provisional subscription is expired");
    }
    elsif($status eq "expired")
    {
        return $self->__("Subscription is expired");
    }
    elsif($status eq "active")
    {
        return $self->__("Subscription is active");
    }
    else
    {
        return "$status ($type)";
    }
}

sub __
{
    my $self = shift;
    my $msgid = shift;
    my $domain = "subscription-tools";
    return Locale::gettext::dgettext ($domain, $msgid);
}

sub __N
{
    my $self = shift;
    my ($msgid, $msgidpl, $n) = @_;
    my $domain = "subscription-tools";
    return Locale::gettext::dngettext ($domain, $msgid, $msgidpl, $n);
}

sub __NOOP
{
    my $self = shift;
    return shift;
}

sub __parseStatus
{
    my $self = shift;
    my $statusfile = shift;

    if( -s $statusfile )
    {
        $self->{registration_status} = undef;
        eval
        {
            my $xml = new XML::Simple;
            $self->{registration_status} = $xml->XMLin($statusfile, ForceArray => ["productstatus"]);
        };
        if($@)
        {
            # no chance to get status info; set to undef
            $self->{registration_status} = undef;
        }
    }
}

sub __parseSCCStatus
{
    my $self = shift;
    my $status = shift;

    if ( $status !~ /^\[/ )
    {
        # The status is and error and not JSON
        return;
    }
    my $json = JSON::decode_json( $status );
    $self->{registration_status}->{generated} = time;
    $self->{registration_status}->{productstatus} = [];
    foreach my $prd (@$json)
    {
        my $product = {};
        $product->{product} = $prd->{identifier} || '';
        $product->{version} = $prd->{version} || '';
        $product->{arch} = $prd->{arch} || '';
        $product->{release} = $prd->{release_type} || '';
        $product->{message} = $prd->{status} || 'Not Registered';
        $product->{result} = (($product->{message} eq 'Registered')?'success':'error');
        $product->{errorcode} = (($product->{message} eq 'Registered')?'OK':'ERR_NOT_REGISTERED');
        if (exists $prd->{subscription_status})
        {
            $product->{subscription} = {
                'type' => $prd->{type},
                'status' => $prd->{subscription_status},
                'expiration' => Date::Parse::str2time($prd->{expires_at})
            };
        }

        push @{$self->{registration_status}->{productstatus}}, $product;
    }
}

=back

=head1 AUTHORS

Duncan Mac-Vicar Prett, Michael Calmer

=head1 LICENSE

Copyright 2011-2014 SUSE LINUX Products GmbH, Nuernberg, Germany.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=cut

1;
