package YaPI::SubscriptionTools;
#
# Copyright (c) 2011-2014 SUSE Linux Products GmbH
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.

BEGIN {
    push @INC, '/usr/share/YaST2/modules/';
}

use strict;
use vars qw(@ISA);
use YaST::YCP qw(Boolean);
use ycp;
use YaPI;
use SUSE::Corral;
use Data::Dumper;
textdomain("subscription-tools");

@YaPI::SubscriptionTools::ISA = qw( YaPI );

our %TYPEINFO;

BEGIN { $TYPEINFO{isCompliant} = ["function",
                                  ["map", "string", "any" ],
                                  ["list", "string"], ["list", ["map", "string", "string"] ], ["list", "string"]
                                 ]; }
sub isCompliant
{
    my $self     = shift;
    my $profiles = shift || undef;
    my $products = shift || undef;
    my $sigkeys  = shift || undef;
    my $debug = 0;
    $debug = 1 if( $ENV{Y2DEBUG} );

    my $corral = SUSE::Corral->new(verbose => 1,
                                   debug => $debug,
                                   profiles => $profiles,
                                   products => $products,
                                   sigkeys => $sigkeys);
    my $err = $corral->checkRules();
    my $res = $corral->getResults();
    y2milestone("RESULT: ".Data::Dumper->Dump([$res]));
    if(!$res || ref($res) ne "HASH" || !(scalar %{$res}))
    {
        return undef;
    }
    return $res;
}

1;

=head1 NAME

YaPI::SubscriptionTools

=head1 PREFACE

This package is the public Yast2 API for the subscription tools.

=head1 SYNOPSIS

use YaPI::SubscriptionTools

$res = YaPI::SubscriptionTools->isCompliant($profiles, $products, $sigkeys)

  Checks the profile compliance with the system.

=head1 DESCRIPTION

This package provide YaST compatible API to check system compliance.

=head1 METHODS

=over 4

=item isCompliant($profiles, $products, $sigkeys)

$profiles, $products and $sigkey are optional parameter. If they are set to undef,
the profiles, products and keys are read from the installed system.

$profiles and $sigkeys are lists of absolute paths to the files.

$products is a list of maps which describe the installed products or the products
which should be installed. The required key for a product
are B<name>, B<version>, B<release>, B<arch> and B<vendor>

If the system is compliant to the profile, the function returns
I<undef>. Otherwise it returns a map.

Example output:

  $VAR1 = {
          'action' => 'warn',
          'arch' => {
                    'oneof' => [
                                 's390x',
                                 'i586'
                               ],
                    'type' => 'enum',
                    'is' => 'x86_64',
                    'message' => 'Architecture(x86_64) is not one of: s390x, i586'
                  },
          'cpu-cores' => {
                         'max' => 1,
                         'type' => 'range',
                         'is' => 2,
                         'message' => 'Number of CPU cores(2) exceeds 1'
                       },
          'ram' => {
                   'max' => 3597436,
                   'type' => 'range',
                   'is' => 5992720,
                   'message' => 'Amount of RAM(5992720) exceeds 3597436'
                 }
        };

Code Example:

 my $profiles = ['/path/to/profile1', '/path/to/profile2'];
 my $products = [
            { 'name'    => 'SUSE_SLES',
              'version' => '11.1',
              'release' => '1.152',
              'arch'    => 'x86_64',
              'vendor'  => 'SUSE LINUX Products GmbH, Nuernberg, Germany'
            },
            { 'name'    => 'sle-hae',
              'version' => '11.1',
              'release' => '1.58',
              'arch'    => 'x86_64',
              'vendor'  => 'SUSE LINUX Products GmbH, Nuernberg, Germany'
            }
 ];
 my $keys = [ '/path/to/sig.key' ];
 my $res = YaPI::SubscriptionTools->isCompliant($profiles, $product, $keys);
 if(not defined $res) {
     # system is compliant
     return 0;
 }
 # we have an error
 print STDERR "System is not compliant.";
 foreach my $set (values %{$res})
 {
     next if(ref($set) ne "HASH");
     print STDERR " * ".$set->{message}."\n";
 }

=back

=head1 AUTHORS

Duncan Mac-Vicar Prett, Michael Calmer

=head1 LICENSE

Copyright 2011-2014 SUSE LINUX Products GmbH, Nuernberg, Germany.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=cut
